@@ -1,4 +1,5 @@
 from functools import partial
+from random import randrange
 
 import torch
 from torch import nn
@@ -14,13 +15,22 @@ class ResidualVQ(nn.Module):
         num_quantizers,
         shared_codebook = False,
         heads = 1,
+        quantize_dropout = False,
+        quantize_dropout_cutoff_index = 0,
         **kwargs
     ):
         super().__init__()
         assert heads == 1, 'residual vq is not compatible with multi-headed codes'
 
+        self.num_quantizers = num_quantizers
+
         self.layers = nn.ModuleList([VectorQuantize(**kwargs) for _ in range(num_quantizers)])
 
+        self.quantize_dropout = quantize_dropout
+
+        assert quantize_dropout_cutoff_index >= 0
+        self.quantize_dropout_cutoff_index = quantize_dropout_cutoff_index
+
         if not shared_codebook:
             return
 
@@ -42,7 +52,14 @@ class ResidualVQ(nn.Module):
         codebooks = repeat(self.codebooks, 'q c d -> q b c d', b = batch)
         gather_indices = repeat(indices, 'b n q -> q b n d', d = codebooks.shape[-1])
 
+        # take care of quantizer dropout
+        mask = gather_indices == -1.
+        gather_indices = gather_indices.masked_fill(mask, 0) # have it fetch a dummy code to be masked out later
+
         all_codes = codebooks.gather(2, gather_indices) # gather all codes
+
+        # mask out any codes that were dropout-ed
+        all_codes = all_codes.masked_fill(mask, 0.)
         return all_codes
 
     def forward(
@@ -50,13 +67,27 @@ class ResidualVQ(nn.Module):
         x,
         return_all_codes = False
     ):
+        b, n, *_, num_quant, device = *x.shape, self.num_quantizers, x.device
+
         quantized_out = 0.
         residual = x
 
         all_losses = []
         all_indices = []
 
-        for layer in self.layers:
+        if self.training and self.quantize_dropout:
+            rand_quantize_dropout_index = randrange(self.quantize_dropout_cutoff_index, num_quant)
+
+        for quantizer_index, layer in enumerate(self.layers):
+
+            if self.training and quantizer_index > rand_quantize_dropout_index:
+                null_indices = torch.full((b, n), -1., device = device, dtype = torch.long)
+                null_loss = torch.full((b,), 0., device = device, dtype = x.dtype)
+
+                all_indices.append(null_indices)
+                all_losses.append(null_loss)
+                continue
+
             quantized, indices, loss = layer(residual)
             residual = residual - quantized
             quantized_out = quantized_out + quantized
